@@ -11,6 +11,7 @@ pub(crate) enum OperationType {
     RestoreActiveBlob = 2,
     ForceUpdateActiveBlob = 3,
     TryDumpBlobIndexes = 4,
+    TryUpdateActiveBlob = 5,
 }
 
 #[derive(Debug)]
@@ -108,6 +109,11 @@ impl<K: Key + 'static> Observer<K> {
             .await
     }
 
+    pub(crate) async fn try_update_active_blob(&self) {
+        self.send_msg(Msg::new(OperationType::TryUpdateActiveBlob, None))
+            .await
+    }
+
     async fn send_msg(&self, msg: Msg) {
         if let Some(sender) = &self.sender {
             let optype = msg.optype.clone();
